@@ -14,7 +14,7 @@
 #ifndef MLPACK_CORE_KERNELS_LINEAR_KERNEL_HPP
 #define MLPACK_CORE_KERNELS_LINEAR_KERNEL_HPP
 
-#include <mlpack/core.hpp>
+#include <mlpack/prereqs.hpp>
 
 namespace mlpack {
 namespace kernel {
