@@ -5,8 +5,6 @@
  *
  * The implementation of the L_BFGS optimizer.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -228,15 +226,7 @@ bool L_BFGS<FunctionType>::LineSearch(double& functionValue,
     const bool cond2 = (stepSize > maxStep);
     const bool cond3 = (numIterations >= maxLineSearchTrials);
     if (cond1 || cond2 || cond3)
-    {
-      if (cond1)
-        Log::Debug << "stepSize < minStep" << std::endl;
-      if (cond2)
-        Log::Debug << "stepSize > maxStep" << std::endl;
-      if (cond3)
-        Log::Debug << "numIterations >= maxLineSearchTrials (stepSize=" << stepSize << ")" << std::endl;
       break;
-    }
 
     // Scale the step size.
     stepSize *= width;
@@ -410,6 +400,15 @@ double L_BFGS<FunctionType>::Optimize(arma::mat& iterate,
       break;
     }
 
+    // Break if the objective is not a number.
+    if (std::isnan(functionValue))
+    {
+      Log::Warn << "L-BFGS terminated with objective " << functionValue << "; "
+          << "are the objective and gradient functions implemented correctly?"
+          << std::endl;
+      break;
+    }
+
     // Choose the scaling factor.
     double scalingFactor = ChooseScalingFactor(itNum, gradient);
 
@@ -438,7 +437,7 @@ double L_BFGS<FunctionType>::Optimize(arma::mat& iterate,
     }
 
     // If we can't make progress on the gradient, then we'll also accept
-    // a stable function value
+    // a stable function value.
     const double denom =
       std::max(
         std::max(fabs(prevFunctionValue), fabs(functionValue)),
