@@ -6,8 +6,6 @@
  * provides information about tree types.  If you create a tree class, you
  * should specialize this class with the characteristics of your tree.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -86,7 +84,12 @@ class TreeTraits
   static const bool HasOverlappingChildren = true;
 
   /**
-   * This is true if Point(0) is the centroid of the node.
+   * This is true if a point can be included in more than one node.
+   */
+  static const bool HasDuplicatedPoints = false;
+
+  /**
+   * This is true if the first point of each node is the centroid of its bound.
    */
   static const bool FirstPointIsCentroid = false;
 
@@ -105,6 +108,12 @@ class TreeTraits
    * This is true if the tree always has only two children.
    */
   static const bool BinaryTree = false;
+
+  /**
+   * This is true if the NumDescendants() method doesn't include duplicated
+   * points.
+   */
+  static const bool UniqueNumDescendants = true;
 };
 
 } // namespace tree
