@@ -4,8 +4,6 @@
  *
  * Definition of the CLIDeleter() class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -31,10 +29,7 @@ class CLIDeleter
   ~CLIDeleter();
 };
 
-//! Declare the deleter.
-static CLIDeleter cliDeleter;
-
-} // namespace io
+} // namespace util
 } // namespace mlpack
 
 #endif
