@@ -4,8 +4,6 @@
  *
  * Implementation of template functions for the Option class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -24,20 +22,21 @@ namespace util {
  * Registers a parameter with CLI.
  */
 template<typename N>
-Option<N>::Option(bool ignoreTemplate,
-                  N defaultValue,
+Option<N>::Option(const bool ignoreTemplate,
+                  const N defaultValue,
                   const std::string& identifier,
                   const std::string& description,
                   const std::string& alias,
-                  bool required)
+                  const bool required,
+                  const bool input)
 {
   if (ignoreTemplate)
   {
-    CLI::Add(identifier, description, alias, required);
+    CLI::Add(identifier, description, alias, required, input);
   }
   else
   {
-    CLI::Add<N>(identifier, description, alias, required);
+    CLI::Add<N>(identifier, description, alias, required, input);
     CLI::GetParam<N>(identifier) = defaultValue;
   }
 }
