@@ -12,7 +12,8 @@
 #ifndef MLPACK_METHODS_LMF_RANDOM_ACOL_INIT_HPP
 #define MLPACK_METHODS_LMF_RANDOM_ACOL_INIT_HPP
 
-#include <mlpack/core.hpp>
+#include <mlpack/prereqs.hpp>
+#include <mlpack/core/math/random.hpp>
 
 namespace mlpack {
 namespace amf {
