@@ -13,7 +13,7 @@
 #ifndef MLPACK_METHODS_LMF_RANDOM_INIT_HPP
 #define MLPACK_METHODS_LMF_RANDOM_INIT_HPP
 
-#include <mlpack/core.hpp>
+#include <mlpack/prereqs.hpp>
 
 namespace mlpack {
 namespace amf {
