@@ -10,6 +10,8 @@
  * http://www.opensource.org/licenses/BSD-3-Clause for more information.
  */
 #include "lars.hpp"
+#include <mlpack/core/util/log.hpp>
+#include <mlpack/core/util/timers.hpp>
 
 using namespace mlpack;
 using namespace mlpack::regression;
