@@ -12,7 +12,7 @@
 #ifndef MLPACK_METHODS_NCA_NCA_HPP
 #define MLPACK_METHODS_NCA_NCA_HPP
 
-#include <mlpack/core.hpp>
+#include <mlpack/prereqs.hpp>
 #include <mlpack/core/metrics/lmetric.hpp>
 #include <mlpack/core/optimizers/sgd/sgd.hpp>
 
