@@ -13,7 +13,7 @@
 #ifndef MLPACK_METHODS_NEIGHBOR_SEARCH_NEIGHBOR_SEARCH_HPP
 #define MLPACK_METHODS_NEIGHBOR_SEARCH_NEIGHBOR_SEARCH_HPP
 
-#include <mlpack/core.hpp>
+#include <mlpack/prereqs.hpp>
 #include <vector>
 #include <string>
 
@@ -21,7 +21,6 @@
 #include <mlpack/core/tree/rectangle_tree.hpp>
 #include <mlpack/core/tree/binary_space_tree/binary_space_tree.hpp>
 
-#include <mlpack/core/metrics/lmetric.hpp>
 #include "neighbor_search_stat.hpp"
 #include "sort_policies/nearest_neighbor_sort.hpp"
 #include "neighbor_search_rules.hpp"
@@ -310,6 +309,16 @@ class NeighborSearch
                                    const double epsilon = 0,
                                    const MetricType metric = MetricType());
 
+  /**
+   * Copy the given NeighborSearch object (this may consume a lot of memory; be
+   * careful!).
+   */
+  NeighborSearch(const NeighborSearch& other);
+
+  /**
+   * Take possession of the given NeighborSearch object.
+   */
+  NeighborSearch(NeighborSearch&& other);
 
   /**
    * Delete the NeighborSearch object. The tree is the only member we are
