@@ -5,8 +5,6 @@
  * Defines the pruning rules and base case rules necessary to perform a
  * tree-based search (with an arbitrary tree) for the NeighborSearch class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -20,19 +18,52 @@
 namespace mlpack {
 namespace neighbor {
 
+/**
+ * The NeighborSearchRules class is a template helper class used by
+ * NeighborSearch class when performing distance-based neighbor searches.  For
+ * each point in the query dataset, it keeps track of the k neighbors in the
+ * reference dataset which have the 'best' distance according to a given sorting
+ * policy.
+ *
+ * @tparam SortPolicy The sort policy for distances.
+ * @tparam MetricType The metric to use for computation.
+ * @tparam TreeType The tree type to use; must adhere to the TreeType API.
+ */
 template<typename SortPolicy, typename MetricType, typename TreeType>
 class NeighborSearchRules
 {
  public:
+  /**
+   * Construct the NeighborSearchRules object.  This is usually done from within
+   * the NeighborSearch class at search time.
+   *
+   * @param referenceSet Set of reference data.
+   * @param querySet Set of query data.
+   * @param k Number of neighbors to search for.
+   * @param metric Instantiated metric.
+   * @param epsilon Relative approximate error.
+   * @param sameSet If true, the query and reference set are taken to be the
+   *      same, and a query point will not return itself in the results.
+   */
   NeighborSearchRules(const typename TreeType::Mat& referenceSet,
                       const typename TreeType::Mat& querySet,
-                      arma::Mat<size_t>& neighbors,
-                      arma::mat& distances,
+                      const size_t k,
                       MetricType& metric,
+                      const double epsilon = 0,
                       const bool sameSet = false);
+
+  /**
+   * Store the list of candidates for each query point in the given matrices.
+   *
+   * @param neighbors Matrix storing lists of neighbors for each query point.
+   * @param distances Matrix storing distances of neighbors for each query
+   *     point.
+   */
+  void GetResults(arma::Mat<size_t>& neighbors, arma::mat& distances);
+
   /**
    * Get the distance from the query point to the reference point.
-   * This will update the "neighbor" matrix with the new point if appropriate
+   * This will update the list of candidates with the new point if appropriate
    * and will track the number of base cases (number of points evaluated).
    *
    * @param queryIndex Index of query point.
@@ -50,6 +81,22 @@ class NeighborSearchRules
    */
   double Score(const size_t queryIndex, TreeType& referenceNode);
 
+  /**
+   * Get the child node with the best score.
+   *
+   * @param queryIndex Index of query point.
+   * @param referenceNode Candidate node to be recursed into.
+   */
+  size_t GetBestChild(const size_t queryIndex, TreeType& referenceNode);
+
+  /**
+   * Get the child node with the best score.
+   *
+   * @param queryNode Node to be considered.
+   * @param referenceNode Candidate node to be recursed into.
+   */
+  size_t GetBestChild(const TreeType& queryNode, TreeType& referenceNode);
+
   /**
    * Re-evaluate the score for recursion order.  A low score indicates priority
    * for recursion, while DBL_MAX indicates that the node should not be recursed
@@ -115,11 +162,26 @@ class NeighborSearchRules
   //! The query set.
   const typename TreeType::Mat& querySet;
 
-  //! The matrix the resultant neighbor indices should be stored in.
-  arma::Mat<size_t>& neighbors;
+  //! Candidate represents a possible candidate neighbor (distance, index).
+  typedef std::pair<double, size_t> Candidate;
 
-  //! The matrix the resultant neighbor distances should be stored in.
-  arma::mat& distances;
+  //! Compare two candidates based on the distance.
+  struct CandidateCmp {
+    bool operator()(const Candidate& c1, const Candidate& c2)
+    {
+      return !SortPolicy::IsBetter(c2.first, c1.first);
+    };
+  };
+
+  //! Use a priority queue to represent the list of candidate neighbors.
+  typedef std::priority_queue<Candidate, std::vector<Candidate>, CandidateCmp>
+      CandidateList;
+
+  //! Set of candidate neighbors for each point.
+  std::vector<CandidateList> candidates;
+
+  //! Number of neighbors to search for.
+  const size_t k;
 
   //! The instantiated metric.
   MetricType& metric;
@@ -127,6 +189,9 @@ class NeighborSearchRules
   //! Denotes whether or not the reference and query sets are the same.
   bool sameSet;
 
+  //! Relative error to be considered in approximate search.
+  const double epsilon;
+
   //! The last query point BaseCase() was called with.
   size_t lastQueryIndex;
   //! The last reference point BaseCase() was called with.
@@ -149,16 +214,13 @@ class NeighborSearchRules
   double CalculateBound(TreeType& queryNode) const;
 
   /**
-   * Insert a point into the neighbors and distances matrices; this is a helper
-   * function.
+   * Helper function to insert a point into the list of candidate points.
    *
    * @param queryIndex Index of point whose neighbors we are inserting into.
-   * @param pos Position in list to insert into.
    * @param neighbor Index of reference point which is being inserted.
    * @param distance Distance from query point to reference point.
    */
   void InsertNeighbor(const size_t queryIndex,
-                      const size_t pos,
                       const size_t neighbor,
                       const double distance);
 };
