@@ -1,10 +1,8 @@
 /**
- * @file nearest_neighbor_rules_impl.hpp
+ * @file neighbor_search_rules_impl.hpp
  * @author Ryan Curtin
  *
- * Implementation of NearestNeighborRules.
- *
- * This file is part of mlpack 2.0.3.
+ * Implementation of NeighborSearchRules.
  *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
@@ -16,6 +14,7 @@
 
 // In case it hasn't been included yet.
 #include "neighbor_search_rules.hpp"
+#include <mlpack/core/tree/spill_tree/is_spill_tree.hpp>
 
 namespace mlpack {
 namespace neighbor {
@@ -24,16 +23,16 @@ template<typename SortPolicy, typename MetricType, typename TreeType>
 NeighborSearchRules<SortPolicy, MetricType, TreeType>::NeighborSearchRules(
     const typename TreeType::Mat& referenceSet,
     const typename TreeType::Mat& querySet,
-    arma::Mat<size_t>& neighbors,
-    arma::mat& distances,
+    const size_t k,
     MetricType& metric,
+    const double epsilon,
     const bool sameSet) :
     referenceSet(referenceSet),
     querySet(querySet),
-    neighbors(neighbors),
-    distances(distances),
+    k(k),
     metric(metric),
     sameSet(sameSet),
+    epsilon(epsilon),
     lastQueryIndex(querySet.n_cols),
     lastReferenceIndex(referenceSet.n_cols),
     baseCases(0),
@@ -44,8 +43,42 @@ NeighborSearchRules<SortPolicy, MetricType, TreeType>::NeighborSearchRules(
   // use the this pointer.
   traversalInfo.LastQueryNode() = (TreeType*) this;
   traversalInfo.LastReferenceNode() = (TreeType*) this;
+
+  // Let's build the list of candidate neighbors for each query point.
+  // It will be initialized with k candidates: (WorstDistance, size_t() - 1)
+  // The list of candidates will be updated when visiting new points with the
+  // BaseCase() method.
+  const Candidate def = std::make_pair(SortPolicy::WorstDistance(),
+      size_t() - 1);
+
+  std::vector<Candidate> vect(k, def);
+  CandidateList pqueue(CandidateCmp(), std::move(vect));
+
+  candidates.reserve(querySet.n_cols);
+  for (size_t i = 0; i < querySet.n_cols; i++)
+    candidates.push_back(pqueue);
 }
 
+template<typename SortPolicy, typename MetricType, typename TreeType>
+void NeighborSearchRules<SortPolicy, MetricType, TreeType>::GetResults(
+    arma::Mat<size_t>& neighbors,
+    arma::mat& distances)
+{
+  neighbors.set_size(k, querySet.n_cols);
+  distances.set_size(k, querySet.n_cols);
+
+  for (size_t i = 0; i < querySet.n_cols; i++)
+  {
+    CandidateList& pqueue = candidates[i];
+    for (size_t j = 1; j <= k; j++)
+    {
+      neighbors(k - j, i) = pqueue.top().second;
+      distances(k - j, i) = pqueue.top().first;
+      pqueue.pop();
+    }
+  }
+};
+
 template<typename SortPolicy, typename MetricType, typename TreeType>
 inline force_inline // Absolutely MUST be inline so optimizations can happen.
 double NeighborSearchRules<SortPolicy, MetricType, TreeType>::
@@ -64,16 +97,7 @@ BaseCase(const size_t queryIndex, const size_t referenceIndex)
                                     referenceSet.col(referenceIndex));
   ++baseCases;
 
-  // If this distance is better than any of the current candidates, the
-  // SortDistance() function will give us the position to insert it into.
-  arma::vec queryDist = distances.unsafe_col(queryIndex);
-  arma::Col<size_t> queryIndices = neighbors.unsafe_col(queryIndex);
-  const size_t insertPosition = SortPolicy::SortDistance(queryDist,
-      queryIndices, distance);
-
-  // SortDistance() returns (size_t() - 1) if we shouldn't add it.
-  if (insertPosition != (size_t() - 1))
-    InsertNeighbor(queryIndex, insertPosition, referenceIndex, distance);
+  InsertNeighbor(queryIndex, referenceIndex, distance);
 
   // Cache this information for the next time BaseCase() is called.
   lastQueryIndex = queryIndex;
@@ -119,9 +143,27 @@ inline double NeighborSearchRules<SortPolicy, MetricType, TreeType>::Score(
   }
 
   // Compare against the best k'th distance for this query point so far.
-  const double bestDistance = distances(distances.n_rows - 1, queryIndex);
+  double bestDistance = candidates[queryIndex].top().first;
+  bestDistance = SortPolicy::Relax(bestDistance, epsilon);
 
-  return (SortPolicy::IsBetter(distance, bestDistance)) ? distance : DBL_MAX;
+  return (SortPolicy::IsBetter(distance, bestDistance)) ?
+      SortPolicy::ConvertToScore(distance) : DBL_MAX;
+}
+
+template<typename SortPolicy, typename MetricType, typename TreeType>
+inline size_t NeighborSearchRules<SortPolicy, MetricType, TreeType>::
+GetBestChild(const size_t queryIndex, TreeType& referenceNode)
+{
+  ++scores;
+  return SortPolicy::GetBestChild(querySet.col(queryIndex), referenceNode);
+}
+
+template<typename SortPolicy, typename MetricType, typename TreeType>
+inline size_t NeighborSearchRules<SortPolicy, MetricType, TreeType>::
+GetBestChild(const TreeType& queryNode, TreeType& referenceNode)
+{
+  ++scores;
+  return SortPolicy::GetBestChild(queryNode, referenceNode);
 }
 
 template<typename SortPolicy, typename MetricType, typename TreeType>
@@ -134,10 +176,13 @@ inline double NeighborSearchRules<SortPolicy, MetricType, TreeType>::Rescore(
   if (oldScore == DBL_MAX)
     return oldScore;
 
-  // Just check the score again against the distances.
-  const double bestDistance = distances(distances.n_rows - 1, queryIndex);
+  const double distance = SortPolicy::ConvertToDistance(oldScore);
 
-  return (SortPolicy::IsBetter(oldScore, bestDistance)) ? oldScore : DBL_MAX;
+  // Just check the score again against the distances.
+  double bestDistance = candidates[queryIndex].top().first;
+  bestDistance = SortPolicy::Relax(bestDistance, epsilon);
+
+  return (SortPolicy::IsBetter(distance, bestDistance)) ? oldScore : DBL_MAX;
 }
 
 template<typename SortPolicy, typename MetricType, typename TreeType>
@@ -290,7 +335,7 @@ inline double NeighborSearchRules<SortPolicy, MetricType, TreeType>::Score(
     traversalInfo.LastReferenceNode() = &referenceNode;
     traversalInfo.LastScore() = distance;
 
-    return distance;
+    return SortPolicy::ConvertToScore(distance);
   }
   else
   {
@@ -307,13 +352,15 @@ inline double NeighborSearchRules<SortPolicy, MetricType, TreeType>::Rescore(
     TreeType& /* referenceNode */,
     const double oldScore) const
 {
-  if (oldScore == DBL_MAX)
+  if (oldScore == DBL_MAX || oldScore == 0.0)
     return oldScore;
 
+  const double distance = SortPolicy::ConvertToDistance(oldScore);
+
   // Update our bound.
   const double bestDistance = CalculateBound(queryNode);
 
-  return (SortPolicy::IsBetter(oldScore, bestDistance)) ? oldScore : DBL_MAX;
+  return (SortPolicy::IsBetter(distance, bestDistance)) ? oldScore : DBL_MAX;
 }
 
 // Calculate the bound for a given query node in its current state and update
@@ -357,7 +404,7 @@ inline double NeighborSearchRules<SortPolicy, MetricType, TreeType>::
   // Loop over points held in the node.
   for (size_t i = 0; i < queryNode.NumPoints(); ++i)
   {
-    const double distance = distances(distances.n_rows - 1, queryNode.Point(i));
+    const double distance = candidates[queryNode.Point(i)].top().first;
     if (SortPolicy::IsBetter(worstDistance, distance))
       worstDistance = distance;
     if (SortPolicy::IsBetter(distance, bestPointDistance))
@@ -426,6 +473,12 @@ inline double NeighborSearchRules<SortPolicy, MetricType, TreeType>::
   queryNode.Stat().SecondBound() = bestDistance;
   queryNode.Stat().AuxBound() = auxDistance;
 
+  worstDistance = SortPolicy::Relax(worstDistance, epsilon);
+
+  // We can't consider B_2 for Spill Trees.
+  if (tree::IsSpillTree<TreeType>::value)
+    return worstDistance;
+
   if (SortPolicy::IsBetter(worstDistance, bestDistance))
     return worstDistance;
   else
@@ -433,35 +486,27 @@ inline double NeighborSearchRules<SortPolicy, MetricType, TreeType>::
 }
 
 /**
- * Helper function to insert a point into the neighbors and distances matrices.
+ * Helper function to insert a point into the list of candidate points.
  *
  * @param queryIndex Index of point whose neighbors we are inserting into.
- * @param pos Position in list to insert into.
  * @param neighbor Index of reference point which is being inserted.
  * @param distance Distance from query point to reference point.
  */
 template<typename SortPolicy, typename MetricType, typename TreeType>
-void NeighborSearchRules<SortPolicy, MetricType, TreeType>::InsertNeighbor(
+inline void NeighborSearchRules<SortPolicy, MetricType, TreeType>::
+InsertNeighbor(
     const size_t queryIndex,
-    const size_t pos,
     const size_t neighbor,
     const double distance)
 {
-  // We only memmove() if there is actually a need to shift something.
-  if (pos < (distances.n_rows - 1))
-  {
-    int len = (distances.n_rows - 1) - pos;
-    memmove(distances.colptr(queryIndex) + (pos + 1),
-        distances.colptr(queryIndex) + pos,
-        sizeof(double) * len);
-    memmove(neighbors.colptr(queryIndex) + (pos + 1),
-        neighbors.colptr(queryIndex) + pos,
-        sizeof(size_t) * len);
-  }
+  CandidateList& pqueue = candidates[queryIndex];
+  Candidate c = std::make_pair(distance, neighbor);
 
-  // Now put the new information in the right index.
-  distances(pos, queryIndex) = distance;
-  neighbors(pos, queryIndex) = neighbor;
+  if (CandidateCmp()(c, pqueue.top()))
+  {
+    pqueue.pop();
+    pqueue.push(c);
+  }
 }
 
 } // namespace neighbor
