@@ -5,8 +5,6 @@
  * Implementation of templated methods for the NearestNeighborSort SortPolicy
  * class for the NeighborSearch class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -25,7 +23,7 @@ inline double NearestNeighborSort::BestNodeToNodeDistance(
 {
   // This is not implemented yet for the general case because the trees do not
   // accept arbitrary distance metrics.
-  return queryNode->MinDistance(referenceNode);
+  return queryNode->MinDistance(*referenceNode);
 }
 
 template<typename TreeType>
@@ -34,7 +32,7 @@ inline double NearestNeighborSort::BestNodeToNodeDistance(
     const TreeType* referenceNode,
     const double centerToCenterDistance)
 {
-  return queryNode->MinDistance(referenceNode, centerToCenterDistance);
+  return queryNode->MinDistance(*referenceNode, centerToCenterDistance);
 }
 
 template<typename TreeType>
@@ -44,7 +42,7 @@ inline double NearestNeighborSort::BestNodeToNodeDistance(
     const TreeType* referenceChildNode,
     const double centerToCenterDistance)
 {
-  return queryNode->MinDistance(referenceChildNode, centerToCenterDistance) -
+  return queryNode->MinDistance(*referenceChildNode, centerToCenterDistance) -
       referenceChildNode->ParentDistance();
 }
 
