@@ -6,8 +6,6 @@
  * class which are commonly used.  This is meant to be included by
  * neighbor_search.h but is a separate file for simplicity.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -39,6 +37,35 @@ typedef NeighborSearch<NearestNeighborSort, metric::EuclideanDistance> KNN;
  */
 typedef NeighborSearch<FurthestNeighborSort, metric::EuclideanDistance> KFN;
 
+/**
+ * The DefeatistKNN class is the k-nearest-neighbors method considering
+ * defeatist search. It returns L2 distances (Euclidean distances) for each of
+ * the k nearest neighbors found.
+ * @tparam TreeType The tree type to use; must adhere to the TreeType API,
+ *     and implement Defeatist Traversers.
+ */
+template<template<typename TreeMetricType,
+                  typename TreeStatType,
+                  typename TreeMatType> class TreeType = tree::SPTree>
+using DefeatistKNN = NeighborSearch<
+    NearestNeighborSort,
+    metric::EuclideanDistance,
+    arma::mat,
+    TreeType,
+    TreeType<metric::EuclideanDistance,
+        NeighborSearchStat<NearestNeighborSort>,
+        arma::mat>::template DefeatistDualTreeTraverser,
+    TreeType<metric::EuclideanDistance,
+        NeighborSearchStat<NearestNeighborSort>,
+        arma::mat>::template DefeatistSingleTreeTraverser>;
+
+/**
+ * The SpillKNN class is the k-nearest-neighbors method considering defeatist
+ * search on SPTree.  It returns L2 distances (Euclidean distances) for each of
+ * the k nearest neighbors found.
+ */
+typedef DefeatistKNN<tree::SPTree> SpillKNN;
+
 /**
  * @deprecated
  * The AllkNN class is the k-nearest-neighbors method.  It returns L2 distances
