@@ -6,8 +6,6 @@
  * easy way to serialize a rank-approximate neighbor search model by abstracting
  * the types of trees and reflecting the RASearch API.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -19,6 +17,7 @@
 #include <mlpack/core/tree/binary_space_tree.hpp>
 #include <mlpack/core/tree/cover_tree.hpp>
 #include <mlpack/core/tree/rectangle_tree.hpp>
+#include <mlpack/core/tree/octree.hpp>
 
 #include "ra_search.hpp"
 
@@ -47,7 +46,12 @@ class RAModel
     COVER_TREE,
     R_TREE,
     R_STAR_TREE,
-    X_TREE
+    X_TREE,
+    HILBERT_R_TREE,
+    R_PLUS_TREE,
+    R_PLUS_PLUS_TREE,
+    UB_TREE,
+    OCTREE
   };
 
  private:
@@ -80,6 +84,16 @@ class RAModel
   RAType<tree::RStarTree>* rStarTreeRA;
   //! Non-NULL if the X tree is used.
   RAType<tree::XTree>* xTreeRA;
+  //! Non-NULL if the Hilbert R tree is used.
+  RAType<tree::HilbertRTree>* hilbertRTreeRA;
+  //! Non-NULL if the R+ tree is used.
+  RAType<tree::RPlusTree>* rPlusTreeRA;
+  //! Non-NULL if the R++ tree is used.
+  RAType<tree::RPlusPlusTree>* rPlusPlusTreeRA;
+  //! Non-NULL if the UB tree is used.
+  RAType<tree::UBTree>* ubTreeRA;
+  //! Non-NULL if the octree is used.
+  RAType<tree::Octree>* octreeRA;
 
  public:
   /**
