@@ -13,7 +13,7 @@
 #ifndef MLPACK_METHODS_RANN_RA_QUERY_STAT_HPP
 #define MLPACK_METHODS_RANN_RA_QUERY_STAT_HPP
 
-#include <mlpack/core.hpp>
+#include <mlpack/prereqs.hpp>
 
 #include <mlpack/core/tree/binary_space_tree.hpp>
 
