@@ -4,8 +4,6 @@
  *
  * Implementation of RASearchRules.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -24,8 +22,7 @@ template<typename SortPolicy, typename MetricType, typename TreeType>
 RASearchRules<SortPolicy, MetricType, TreeType>::
 RASearchRules(const arma::mat& referenceSet,
               const arma::mat& querySet,
-              arma::Mat<size_t>& neighbors,
-              arma::mat& distances,
+              const size_t k,
               MetricType& metric,
               const double tau,
               const double alpha,
@@ -36,8 +33,7 @@ RASearchRules(const arma::mat& referenceSet,
               const bool sameSet) :
     referenceSet(referenceSet),
     querySet(querySet),
-    neighbors(neighbors),
-    distances(distances),
+    k(k),
     metric(metric),
     sampleAtLeaves(sampleAtLeaves),
     firstLeafExact(firstLeafExact),
@@ -49,7 +45,6 @@ RASearchRules(const arma::mat& referenceSet,
 
   // The rank approximation.
   const size_t n = referenceSet.n_cols;
-  const size_t k = neighbors.n_rows;
   const size_t t = (size_t) std::ceil(tau * (double) n / 100.0);
   if (t < k)
   {
@@ -75,19 +70,53 @@ RASearchRules(const arma::mat& referenceSet,
   Log::Info << "Minimum samples required per query: " << numSamplesReqd <<
     ", sampling ratio: " << samplingRatio << std::endl;
 
-  if (naive) // No tree traversal; just do naive sampling here.
+  // Let's build the list of candidate neighbors for each query point.
+  // It will be initialized with k candidates: (WorstDistance, size_t() - 1)
+  // The list of candidates will be updated when visiting new points with the
+  // BaseCase() method.
+  const Candidate def = std::make_pair(SortPolicy::WorstDistance(),
+      size_t() - 1);
+
+  std::vector<Candidate> vect(k, def);
+  CandidateList pqueue(CandidateCmp(), std::move(vect));
+
+  candidates.reserve(querySet.n_cols);
+  for (size_t i = 0; i < querySet.n_cols; i++)
+    candidates.push_back(pqueue);
+
+  if (naive)// No tree traversal; just do naive sampling here.
   {
     // Sample enough points.
+    arma::uvec distinctSamples;
     for (size_t i = 0; i < querySet.n_cols; ++i)
     {
-      arma::uvec distinctSamples;
-      RAUtil::ObtainDistinctSamples(numSamplesReqd, n, distinctSamples);
+      math::ObtainDistinctSamples(0, n, numSamplesReqd, distinctSamples);
       for (size_t j = 0; j < distinctSamples.n_elem; j++)
         BaseCase(i, (size_t) distinctSamples[j]);
     }
   }
 }
 
+template<typename SortPolicy, typename MetricType, typename TreeType>
+void RASearchRules<SortPolicy, MetricType, TreeType>::GetResults(
+    arma::Mat<size_t>& neighbors,
+    arma::mat& distances)
+{
+  neighbors.set_size(k, querySet.n_cols);
+  distances.set_size(k, querySet.n_cols);
+
+  for (size_t i = 0; i < querySet.n_cols; i++)
+  {
+    CandidateList& pqueue = candidates[i];
+    for (size_t j = 1; j <= k; j++)
+    {
+      neighbors(k - j, i) = pqueue.top().second;
+      distances(k - j, i) = pqueue.top().first;
+      pqueue.pop();
+    }
+  }
+};
+
 template<typename SortPolicy, typename MetricType, typename TreeType>
 inline force_inline
 double RASearchRules<SortPolicy, MetricType, TreeType>::BaseCase(
@@ -102,16 +131,7 @@ double RASearchRules<SortPolicy, MetricType, TreeType>::BaseCase(
   double distance = metric.Evaluate(querySet.unsafe_col(queryIndex),
                                     referenceSet.unsafe_col(referenceIndex));
 
-  // If this distance is better than any of the current candidates, the
-  // SortDistance() function will give us the position to insert it into.
-  arma::vec queryDist = distances.unsafe_col(queryIndex);
-  arma::Col<size_t> queryIndices = neighbors.unsafe_col(queryIndex);
-  size_t insertPosition = SortPolicy::SortDistance(queryDist, queryIndices,
-      distance);
-
-  // SortDistance() returns (size_t() - 1) if we shouldn't add it.
-  if (insertPosition != (size_t() - 1))
-    InsertNeighbor(queryIndex, insertPosition, referenceIndex, distance);
+  InsertNeighbor(queryIndex, referenceIndex, distance);
 
   numSamplesMade[queryIndex]++;
 
@@ -129,7 +149,7 @@ inline double RASearchRules<SortPolicy, MetricType, TreeType>::Score(
   const arma::vec queryPoint = querySet.unsafe_col(queryIndex);
   const double distance = SortPolicy::BestPointToNodeDistance(queryPoint,
       &referenceNode);
-  const double bestDistance = distances(distances.n_rows - 1, queryIndex);
+  const double bestDistance = candidates[queryIndex].top().first;
 
   return Score(queryIndex, referenceNode, distance, bestDistance);
 }
@@ -143,7 +163,7 @@ inline double RASearchRules<SortPolicy, MetricType, TreeType>::Score(
   const arma::vec queryPoint = querySet.unsafe_col(queryIndex);
   const double distance = SortPolicy::BestPointToNodeDistance(queryPoint,
       &referenceNode, baseCaseResult);
-  const double bestDistance = distances(distances.n_rows - 1, queryIndex);
+  const double bestDistance = candidates[queryIndex].top().first;
 
   return Score(queryIndex, referenceNode, distance, bestDistance);
 }
@@ -185,9 +205,8 @@ inline double RASearchRules<SortPolicy, MetricType, TreeType>::Score(
           // Then samplesReqd <= singleSampleLimit.
           // Hence, approximate the node by sampling enough number of points.
           arma::uvec distinctSamples;
-          RAUtil::ObtainDistinctSamples(samplesReqd,
-              referenceNode.NumDescendants(),
-                                distinctSamples);
+          math::ObtainDistinctSamples(0, referenceNode.NumDescendants(),
+              samplesReqd, distinctSamples);
           for (size_t i = 0; i < distinctSamples.n_elem; i++)
             // The counting of the samples are done in the 'BaseCase' function
             // so no book-keeping is required here.
@@ -202,9 +221,8 @@ inline double RASearchRules<SortPolicy, MetricType, TreeType>::Score(
           {
             // Approximate node by sampling enough number of points.
             arma::uvec distinctSamples;
-            RAUtil::ObtainDistinctSamples(samplesReqd,
-                referenceNode.NumDescendants(),
-                                  distinctSamples);
+            math::ObtainDistinctSamples(0, referenceNode.NumDescendants(),
+                samplesReqd, distinctSamples);
             for (size_t i = 0; i < distinctSamples.n_elem; i++)
               // The counting of the samples are done in the 'BaseCase' function
               // so no book-keeping is required here.
@@ -257,7 +275,7 @@ Rescore(const size_t queryIndex,
     return oldScore;
 
   // Just check the score again against the distances.
-  const double bestDistance = distances(distances.n_rows - 1, queryIndex);
+  const double bestDistance = candidates[queryIndex].top().first;
 
   // If this is better than the best distance we've seen so far,
   // maybe there will be something down this node.
@@ -291,8 +309,8 @@ Rescore(const size_t queryIndex,
         // Then, samplesReqd <= singleSampleLimit.  Hence, approximate the node
         // by sampling enough number of points.
         arma::uvec distinctSamples;
-        RAUtil::ObtainDistinctSamples(samplesReqd,
-            referenceNode.NumDescendants(), distinctSamples);
+        math::ObtainDistinctSamples(0, referenceNode.NumDescendants(),
+            samplesReqd, distinctSamples);
         for (size_t i = 0; i < distinctSamples.n_elem; i++)
           // The counting of the samples are done in the 'BaseCase' function so
           // no book-keeping is required here.
@@ -307,8 +325,8 @@ Rescore(const size_t queryIndex,
         {
           // Approximate node by sampling enough points.
           arma::uvec distinctSamples;
-          RAUtil::ObtainDistinctSamples(samplesReqd,
-              referenceNode.NumDescendants(), distinctSamples);
+          math::ObtainDistinctSamples(0, referenceNode.NumDescendants(),
+              samplesReqd, distinctSamples);
           for (size_t i = 0; i < distinctSamples.n_elem; i++)
             // The counting of the samples are done in the 'BaseCase' function
             // so no book-keeping is required here.
@@ -357,7 +375,7 @@ inline double RASearchRules<SortPolicy, MetricType, TreeType>::Score(
 
   for (size_t i = 0; i < queryNode.NumPoints(); i++)
   {
-    const double bound = distances(distances.n_rows - 1, queryNode.Point(i))
+    const double bound = candidates[queryNode.Point(i)].top().first
         + maxDescendantDistance;
     if (bound < pointBound)
       pointBound = bound;
@@ -396,7 +414,7 @@ inline double RASearchRules<SortPolicy, MetricType, TreeType>::Score(
 
   for (size_t i = 0; i < queryNode.NumPoints(); i++)
   {
-    const double bound = distances(distances.n_rows - 1, queryNode.Point(i))
+    const double bound = candidates[queryNode.Point(i)].top().first
         + maxDescendantDistance;
     if (bound < pointBound)
       pointBound = bound;
@@ -490,12 +508,12 @@ inline double RASearchRules<SortPolicy, MetricType, TreeType>::Score(
         {
           // Then samplesReqd <= singleSampleLimit.  Hence, approximate node by
           // sampling enough number of points for every query in the query node.
+          arma::uvec distinctSamples;
           for (size_t i = 0; i < queryNode.NumDescendants(); ++i)
           {
             const size_t queryIndex = queryNode.Descendant(i);
-            arma::uvec distinctSamples;
-            RAUtil::ObtainDistinctSamples(samplesReqd,
-                referenceNode.NumDescendants(), distinctSamples);
+            math::ObtainDistinctSamples(0, referenceNode.NumDescendants(),
+                samplesReqd, distinctSamples);
             for (size_t j = 0; j < distinctSamples.n_elem; j++)
               // The counting of the samples are done in the 'BaseCase' function
               // so no book-keeping is required here.
@@ -520,12 +538,12 @@ inline double RASearchRules<SortPolicy, MetricType, TreeType>::Score(
           {
             // Approximate node by sampling enough number of points for every
             // query in the query node.
+            arma::uvec distinctSamples;
             for (size_t i = 0; i < queryNode.NumDescendants(); ++i)
             {
               const size_t queryIndex = queryNode.Descendant(i);
-              arma::uvec distinctSamples;
-              RAUtil::ObtainDistinctSamples(samplesReqd,
-                  referenceNode.NumDescendants(), distinctSamples);
+              math::ObtainDistinctSamples(0, referenceNode.NumDescendants(),
+                  samplesReqd, distinctSamples);
               for (size_t j = 0; j < distinctSamples.n_elem; j++)
                 // The counting of the samples are done in the 'BaseCase'
                 // function so no book-keeping is required here.
@@ -610,7 +628,7 @@ Rescore(TreeType& queryNode,
 
   for (size_t i = 0; i < queryNode.NumPoints(); i++)
   {
-    const double bound = distances(distances.n_rows - 1, queryNode.Point(i))
+    const double bound = candidates[queryNode.Point(i)].top().first
         + maxDescendantDistance;
     if (bound < pointBound)
       pointBound = bound;
@@ -695,12 +713,12 @@ Rescore(TreeType& queryNode,
       {
         // then samplesReqd <= singleSampleLimit.  Hence, approximate the node
         // by sampling enough points for every query in the query node.
+        arma::uvec distinctSamples;
         for (size_t i = 0; i < queryNode.NumDescendants(); ++i)
         {
           const size_t queryIndex = queryNode.Descendant(i);
-          arma::uvec distinctSamples;
-          RAUtil::ObtainDistinctSamples(samplesReqd,
-              referenceNode.NumDescendants(), distinctSamples);
+          math::ObtainDistinctSamples(0, referenceNode.NumDescendants(),
+              samplesReqd, distinctSamples);
           for (size_t j = 0; j < distinctSamples.n_elem; j++)
             // The counting of the samples are done in the 'BaseCase'
             // function so no book-keeping is required here.
@@ -724,12 +742,12 @@ Rescore(TreeType& queryNode,
         {
           // Approximate node by sampling enough points for every query in the
           // query node.
+          arma::uvec distinctSamples;
           for (size_t i = 0; i < queryNode.NumDescendants(); ++i)
           {
             const size_t queryIndex = queryNode.Descendant(i);
-            arma::uvec distinctSamples;
-            RAUtil::ObtainDistinctSamples(samplesReqd,
-                referenceNode.NumDescendants(), distinctSamples);
+            math::ObtainDistinctSamples(0, referenceNode.NumDescendants(),
+                samplesReqd, distinctSamples);
             for (size_t j = 0; j < distinctSamples.n_elem; j++)
               // The counting of the samples are done in BaseCase() so no
               // book-keeping is required here.
@@ -782,35 +800,27 @@ Rescore(TreeType& queryNode,
 } // Rescore(node, node, oldScore)
 
 /**
- * Helper function to insert a point into the neighbors and distances matrices.
+ * Helper function to insert a point into the list of candidate points.
  *
  * @param queryIndex Index of point whose neighbors we are inserting into.
- * @param pos Position in list to insert into.
  * @param neighbor Index of reference point which is being inserted.
  * @param distance Distance from query point to reference point.
  */
 template<typename SortPolicy, typename MetricType, typename TreeType>
-void RASearchRules<SortPolicy, MetricType, TreeType>::InsertNeighbor(
+inline void RASearchRules<SortPolicy, MetricType, TreeType>::
+InsertNeighbor(
     const size_t queryIndex,
-    const size_t pos,
     const size_t neighbor,
     const double distance)
 {
-  // We only memmove() if there is actually a need to shift something.
-  if (pos < (distances.n_rows - 1))
-  {
-    int len = (distances.n_rows - 1) - pos;
-    memmove(distances.colptr(queryIndex) + (pos + 1),
-        distances.colptr(queryIndex) + pos,
-        sizeof(double) * len);
-    memmove(neighbors.colptr(queryIndex) + (pos + 1),
-        neighbors.colptr(queryIndex) + pos,
-        sizeof(size_t) * len);
-  }
+  CandidateList& pqueue = candidates[queryIndex];
+  Candidate c = std::make_pair(distance, neighbor);
 
-  // Now put the new information in the right index.
-  distances(pos, queryIndex) = distance;
-  neighbors(pos, queryIndex) = neighbor;
+  if (CandidateCmp()(c, pqueue.top()))
+  {
+    pqueue.pop();
+    pqueue.push(c);
+  }
 }
 
 } // namespace neighbor
