@@ -5,8 +5,6 @@
  *
  * Utilities for rank-approximate neighbor search.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -171,19 +169,3 @@ double mlpack::neighbor::RAUtil::SuccessProbability(const size_t n,
     return sum;
   } // For k > 1.
 }
-
-void mlpack::neighbor::RAUtil::ObtainDistinctSamples(
-    const size_t numSamples,
-    const size_t rangeUpperBound,
-    arma::uvec& distinctSamples)
-{
-  // Keep track of the points that are sampled.
-  arma::Col<size_t> sampledPoints;
-  sampledPoints.zeros(rangeUpperBound);
-
-  for (size_t i = 0; i < numSamples; i++)
-    sampledPoints[(size_t) math::RandInt(rangeUpperBound)]++;
-
-  distinctSamples = arma::find(sampledPoints > 0);
-  return;
-}
