@@ -13,7 +13,7 @@
 #ifndef MLPACK_METHODS_SPARSE_CODING_RANDOM_INITIALIZER_HPP
 #define MLPACK_METHODS_SPARSE_CODING_RANDOM_INITIALIZER_HPP
 
-#include <mlpack/core.hpp>
+#include <mlpack/prereqs.hpp>
 
 namespace mlpack {
 namespace sparse_coding {
