@@ -3,8 +3,6 @@
  *
  * Tests for KFN (k-furthest-neighbors).
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -14,7 +12,7 @@
 #include <mlpack/methods/neighbor_search/neighbor_search.hpp>
 #include <mlpack/core/tree/cover_tree.hpp>
 #include <boost/test/unit_test.hpp>
-#include "old_boost_test_definitions.hpp"
+#include "test_tools.hpp"
 
 using namespace mlpack;
 using namespace mlpack::neighbor;
@@ -55,28 +53,30 @@ BOOST_AUTO_TEST_CASE(ExhaustiveSyntheticTest)
   // calculation.  We'll always use 10 neighbors, so set that parameter.
   std::vector<size_t> oldFromNew;
   std::vector<size_t> newFromOld;
-  TreeType* tree = new TreeType(data, oldFromNew, newFromOld, 1);
+  TreeType tree(data, oldFromNew, newFromOld, 1);
+  KFN kfn(std::move(tree));
+
   for (int i = 0; i < 3; i++)
   {
-    KFN* kfn;
-
     switch (i)
     {
       case 0: // Use the dual-tree method.
-        kfn = new KFN(tree, false);
+        kfn.Naive() = false;
+        kfn.SingleMode() = false;
         break;
       case 1: // Use the single-tree method.
-        kfn = new KFN(tree, true);
+        kfn.Naive() = false;
+        kfn.SingleMode() = true;
         break;
       case 2: // Use the naive method.
-        kfn = new KFN(tree->Dataset(), true);
+        kfn.Naive() = true;
         break;
     }
 
     // Now perform the actual calculation.
     arma::Mat<size_t> neighbors;
     arma::mat distances;
-    kfn->Search(10, neighbors, distances);
+    kfn.Search(10, neighbors, distances);
 
     // Now the exhaustive check for correctness.  This will be long.  We must
     // also remember that the distances returned are squared distances.  As a
@@ -324,13 +324,7 @@ BOOST_AUTO_TEST_CASE(ExhaustiveSyntheticTest)
     BOOST_REQUIRE_CLOSE(distances(1, newFromOld[10]), 3.00, 1e-5);
     BOOST_REQUIRE_EQUAL(neighbors(0, newFromOld[10]), newFromOld[4]);
     BOOST_REQUIRE_CLOSE(distances(0, newFromOld[10]), 4.05, 1e-5);
-
-    // Clean the memory.
-    delete kfn;
   }
-
-  // We are responsible for the tree, too.
-  delete tree;
 }
 
 /**
@@ -349,7 +343,7 @@ BOOST_AUTO_TEST_CASE(DualTreeVsNaive1)
 
   KFN kfn(dataset);
 
-  KFN naive(dataset, true);
+  KFN naive(dataset, NAIVE_MODE);
 
   arma::Mat<size_t> neighborsTree;
   arma::mat distancesTree;
@@ -383,7 +377,7 @@ BOOST_AUTO_TEST_CASE(DualTreeVsNaive2)
 
   KFN kfn(dataset);
 
-  KFN naive(dataset, true);
+  KFN naive(dataset, NAIVE_MODE);
 
   arma::Mat<size_t> neighborsTree;
   arma::mat distancesTree;
@@ -415,9 +409,9 @@ BOOST_AUTO_TEST_CASE(SingleTreeVsNaive)
   if (!data::Load("test_data_3_1000.csv", dataset))
     BOOST_FAIL("Cannot load test dataset test_data_3_1000.csv!");
 
-  KFN kfn(dataset, false, true);
+  KFN kfn(dataset, SINGLE_TREE_MODE);
 
-  KFN naive(dataset, true);
+  KFN naive(dataset, NAIVE_MODE);
 
   arma::Mat<size_t> neighborsTree;
   arma::mat distancesTree;
@@ -450,9 +444,9 @@ BOOST_AUTO_TEST_CASE(SingleCoverTreeTest)
       FirstPointIsRoot> tree(data);
 
   NeighborSearch<FurthestNeighborSort, LMetric<2>, arma::mat, StandardCoverTree>
-      coverTreeSearch(&tree, true);
+      coverTreeSearch(std::move(tree), SINGLE_TREE_MODE);
 
-  KFN naive(data, true);
+  KFN naive(data, NAIVE_MODE);
 
   arma::Mat<size_t> coverTreeNeighbors;
   arma::mat coverTreeDistances;
@@ -487,14 +481,14 @@ BOOST_AUTO_TEST_CASE(DualCoverTreeTest)
   typedef CoverTree<LMetric<2, true>, NeighborSearchStat<FurthestNeighborSort>,
       arma::mat, FirstPointIsRoot> TreeType;
 
-  TreeType referenceTree = TreeType(dataset);
+  TreeType referenceTree(dataset);
 
   NeighborSearch<FurthestNeighborSort, LMetric<2, true>, arma::mat,
-      StandardCoverTree> coverTreeSearch(&referenceTree);
+      StandardCoverTree> coverTreeSearch(std::move(referenceTree));
 
   arma::Mat<size_t> coverNeighbors;
   arma::mat coverDistances;
-  coverTreeSearch.Search(&referenceTree, 5, coverNeighbors, coverDistances);
+  coverTreeSearch.Search(dataset, 5, coverNeighbors, coverDistances);
 
   for (size_t i = 0; i < coverNeighbors.n_elem; ++i)
   {
@@ -518,13 +512,13 @@ BOOST_AUTO_TEST_CASE(SingleBallTreeTest)
       arma::mat> TreeType;
   TreeType tree(data);
 
+  KFN naive(tree.Dataset(), NAIVE_MODE);
+
   // BinarySpaceTree modifies data. Use modified data to maintain the
   // correspondence between points in the dataset for both methods. The order of
   // query points in both methods should be same.
   NeighborSearch<FurthestNeighborSort, LMetric<2>, arma::mat, BallTree>
-      ballTreeSearch(&tree, true);
-
-  KFN naive(tree.Dataset(), true);
+      ballTreeSearch(std::move(tree), SINGLE_TREE_MODE);
 
   arma::Mat<size_t> ballTreeNeighbors;
   arma::mat ballTreeDistances;
