@@ -156,7 +156,8 @@ BOOST_AUTO_TEST_CASE(DiscreteDistributionTest)
   // straightforward.
   vec prob;
   prob.randu(12);
-  DiscreteDistribution t(prob);
+  std::vector<arma::vec> prob_vector = std::vector<arma::vec>(1, prob);
+  DiscreteDistribution t(prob_vector);
 
   DiscreteDistribution xmlT, textT, binaryT;
 
@@ -1499,8 +1500,8 @@ BOOST_AUTO_TEST_CASE(HoeffdingCategoricalSplitTest)
 BOOST_AUTO_TEST_CASE(HoeffdingTreeBeforeSplitTest)
 {
   data::DatasetInfo info(5);
-  info.MapString("0", 2); // Dimension 1 is categorical.
-  info.MapString("1", 2);
+  info.MapString<double>("0", 2); // Dimension 1 is categorical.
+  info.MapString<double>("1", 2);
   HoeffdingTree<> split(info, 2, 0.99, 15000, 1);
 
   // Train for 2 samples.
@@ -1508,14 +1509,14 @@ BOOST_AUTO_TEST_CASE(HoeffdingTreeBeforeSplitTest)
   split.Train(arma::vec("-0.3 0.0 0 0.7 0.8"), 1);
 
   data::DatasetInfo wrongInfo(3);
-  wrongInfo.MapString("1", 1);
+  wrongInfo.MapString<double>("1", 1);
   HoeffdingTree<> xmlSplit(wrongInfo, 7, 0.1, 10, 1);
 
   // Force the binarySplit to split.
   data::DatasetInfo binaryInfo(2);
-  binaryInfo.MapString("cat0", 0);
-  binaryInfo.MapString("cat1", 0);
-  binaryInfo.MapString("cat0", 1);
+  binaryInfo.MapString<double>("cat0", 0);
+  binaryInfo.MapString<double>("cat1", 0);
+  binaryInfo.MapString<double>("cat0", 1);
 
   HoeffdingTree<> binarySplit(info, 2, 0.95, 5000, 1);
 
@@ -1551,9 +1552,9 @@ BOOST_AUTO_TEST_CASE(HoeffdingTreeAfterSplitTest)
 {
   // Force the split to split.
   data::DatasetInfo info(2);
-  info.MapString("cat0", 0);
-  info.MapString("cat1", 0);
-  info.MapString("cat0", 1);
+  info.MapString<double>("cat0", 0);
+  info.MapString<double>("cat1", 0);
+  info.MapString<double>("cat0", 1);
 
   HoeffdingTree<> split(info, 2, 0.95, 5000, 1);
 
@@ -1567,12 +1568,12 @@ BOOST_AUTO_TEST_CASE(HoeffdingTreeAfterSplitTest)
   BOOST_REQUIRE_NE(split.SplitDimension(), size_t(-1));
 
   data::DatasetInfo wrongInfo(3);
-  wrongInfo.MapString("1", 1);
+  wrongInfo.MapString<double>("1", 1);
   HoeffdingTree<> xmlSplit(wrongInfo, 7, 0.1, 10, 1);
 
   data::DatasetInfo binaryInfo(5);
-  binaryInfo.MapString("0", 2); // Dimension 2 is categorical.
-  binaryInfo.MapString("1", 2);
+  binaryInfo.MapString<double>("0", 2); // Dimension 2 is categorical.
+  binaryInfo.MapString<double>("1", 2);
   HoeffdingTree<> binarySplit(binaryInfo, 2, 0.99, 15000, 1);
 
   // Train for 2 samples.
@@ -1644,14 +1645,14 @@ BOOST_AUTO_TEST_CASE(HoeffdingTreeTest)
   }
   // Make the features categorical.
   data::DatasetInfo info(2);
-  info.MapString("a", 0);
-  info.MapString("b", 0);
-  info.MapString("c", 0);
-  info.MapString("d", 0);
-  info.MapString("a", 1);
-  info.MapString("b", 1);
-  info.MapString("c", 1);
-  info.MapString("d", 1);
+  info.MapString<double>("a", 0);
+  info.MapString<double>("b", 0);
+  info.MapString<double>("c", 0);
+  info.MapString<double>("d", 0);
+  info.MapString<double>("a", 1);
+  info.MapString<double>("b", 1);
+  info.MapString<double>("c", 1);
+  info.MapString<double>("d", 1);
 
   HoeffdingTree<> tree(dataset, info, labels, 2, false /* no batch mode */);
 
