@@ -4,8 +4,6 @@
  *
  * Tests for each of the implementations of the SortPolicy class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -19,7 +17,7 @@
 #include <mlpack/methods/neighbor_search/sort_policies/furthest_neighbor_sort.hpp>
 
 #include <boost/test/unit_test.hpp>
-#include "old_boost_test_definitions.hpp"
+#include "test_tools.hpp"
 
 using namespace mlpack;
 using namespace mlpack::neighbor;
@@ -63,42 +61,6 @@ BOOST_AUTO_TEST_CASE(NnsIsBetterNotStrict)
   BOOST_WARN(NearestNeighborSort::IsBetter(6.0, 6.0) == true);
 }
 
-/**
- * A simple test case of where to insert when all the values in the list are
- * DBL_MAX.
- */
-BOOST_AUTO_TEST_CASE(NnsSortDistanceAllDblMax)
-{
-  arma::vec list(5);
-  list.fill(DBL_MAX);
-  arma::Col<size_t> indices(5);
-  indices.fill(0);
-
-  // Should be inserted at the head of the list.
-  BOOST_REQUIRE(NearestNeighborSort::SortDistance(list, indices, 5.0) == 0);
-}
-
-/**
- * Another test case, where we are just putting the new value in the middle of
- * the list.
- */
-BOOST_AUTO_TEST_CASE(NnsSortDistance2)
-{
-  arma::vec list(3);
-  list[0] = 0.66;
-  list[1] = 0.89;
-  list[2] = 1.14;
-  arma::Col<size_t> indices(3);
-  indices.fill(0);
-
-  // Run a couple possibilities through.
-  BOOST_REQUIRE(NearestNeighborSort::SortDistance(list, indices, 0.61) == 0);
-  BOOST_REQUIRE(NearestNeighborSort::SortDistance(list, indices, 0.76) == 1);
-  BOOST_REQUIRE(NearestNeighborSort::SortDistance(list, indices, 0.99) == 2);
-  BOOST_REQUIRE(NearestNeighborSort::SortDistance(list, indices, 1.22) ==
-      (size_t() - 1));
-}
-
 /**
  * Very simple sanity check to ensure that bounds are working alright.  We will
  * use a one-dimensional bound for simplicity.
@@ -225,42 +187,6 @@ BOOST_AUTO_TEST_CASE(FnsIsBetterNotStrict)
   BOOST_WARN(FurthestNeighborSort::IsBetter(6.0, 6.0) == true);
 }
 
-/**
- * A simple test case of where to insert when all the values in the list are
- * 0.
- */
-BOOST_AUTO_TEST_CASE(FnsSortDistanceAllZero)
-{
-  arma::vec list(5);
-  list.fill(0);
-  arma::Col<size_t> indices(5);
-  indices.fill(0);
-
-  // Should be inserted at the head of the list.
-  BOOST_REQUIRE(FurthestNeighborSort::SortDistance(list, indices, 5.0) == 0);
-}
-
-/**
- * Another test case, where we are just putting the new value in the middle of
- * the list.
- */
-BOOST_AUTO_TEST_CASE(FnsSortDistance2)
-{
-  arma::vec list(3);
-  list[0] = 1.14;
-  list[1] = 0.89;
-  list[2] = 0.66;
-  arma::Col<size_t> indices(3);
-  indices.fill(0);
-
-  // Run a couple possibilities through.
-  BOOST_REQUIRE(FurthestNeighborSort::SortDistance(list, indices, 1.22) == 0);
-  BOOST_REQUIRE(FurthestNeighborSort::SortDistance(list, indices, 0.93) == 1);
-  BOOST_REQUIRE(FurthestNeighborSort::SortDistance(list, indices, 0.68) == 2);
-  BOOST_REQUIRE(FurthestNeighborSort::SortDistance(list, indices, 0.62) ==
-      (size_t() - 1));
-}
-
 /**
  * Very simple sanity check to ensure that bounds are working alright.  We will
  * use a one-dimensional bound for simplicity.
